/*
 *   Copyright © 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>

#include <spdk/env.h>
#include <spdk/event.h>

#include "mlnx_snap_spdk.h"
#include "config.h"

static struct spdk_thread *g_app_thread;

static void stop_done_cb()
{
    spdk_app_stop(0);
}

static void spdk_emu_app_stop(void)
{
    mlnx_snap_spdk_stop(stop_done_cb);
}

static void spdk_emu_app_start(void *arg)
{
    /*
     * Set signal handler to allow stop on Ctrl+C.
     */
    g_app_thread = spdk_get_thread();
    if (!g_app_thread) {
        SPDK_ERRLOG("Failed to get SPDK thread\n");
        goto err;
    }

    if (mlnx_snap_spdk_start()) {
        SPDK_ERRLOG("Failed to start MLNX_SNAP\n");
        goto err;
    }

    SPDK_NOTICELOG("MLNX_SNAP started successfully\n");
    return;

err:
    spdk_app_stop(-1);
}

int main(int argc, char **argv)
{
    struct spdk_app_opts opts = {0};
    int rc;

    if (mlnx_snap_spdk_init())
        exit(1);

    /* Set default values in opts structure. */
#ifdef HAVE_STRUCT_SPDK_APP_OPTS_OPTS_SIZE
    spdk_app_opts_init(&opts, sizeof(opts));
#else
    spdk_app_opts_init(&opts);
#endif
    opts.name = "mlnx_spdk_emu_app";
    opts.shutdown_cb = spdk_emu_app_stop;

    if ((rc = spdk_app_parse_args(argc, argv, &opts, NULL, NULL, NULL,
                                  NULL)) != SPDK_APP_PARSE_ARGS_SUCCESS) {
        exit(rc);
    }

    rc = spdk_app_start(&opts, spdk_emu_app_start, NULL);
    if (rc) {
        SPDK_ERRLOG("ERROR starting application\n");
    }

    /* Gracefully close out all of the SPDK subsystems. */
    SPDK_NOTICELOG("Exiting...\n");
    spdk_app_fini();
    mlnx_snap_spdk_cleanup();
    return rc;
}
