/*
 *   Copyright © 2021 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "spdk_ext_io.h"
#include "effdma/effdma_domain.h"

int spdk_ext_io_effdma_memory_domain_translate_memory_cb(
        struct spdk_memory_domain *src_domain,
        spdk_ext_io_effdma_ctx_t *effdma_ctx,
        struct spdk_memory_domain *dst_domain,
        struct spdk_memory_domain_translation_ctx *dst_domain_ctx,
        void *addr, size_t len,
        struct spdk_memory_domain_translation_result *result)
{
    int rc;

    if (unlikely(!effdma_ctx->enabled))
       return -1;

#ifdef HAVE_STRUCT_SPDK_MEMORY_DOMAIN_TRANSLATION_RESULT_IOV
    rc = effdma_domain_translate(effdma_ctx->domain, (uint64_t)addr, (uint64_t *)&result->iov.iov_base);

    result->iov.iov_len = len;
    result->iov_count = 1;
#else
    rc = effdma_domain_translate(effdma_ctx->domain, (uint64_t)addr, (uint64_t *)&result->addr);

    result->len = len;
#endif
    result->rdma.lkey = 0;
    result->rdma.rkey = 0;
    result->dst_domain = dst_domain;

    return rc;
}

int spdk_ext_io_effdma_status(struct spdk_bdev *bdev)
{
#ifdef HAVE_SPDK_DMA_DEVICE_TYPE_HOST
    return spdk_ext_io_get_bdev_status(bdev, SPDK_DMA_DEVICE_TYPE_HOST);
#else
    return -ENOTSUP;
#endif
}

void spdk_ext_io_effdma_init(spdk_ext_io_effdma_ctx_t *effdma_ctx, struct effdma_domain *domain)
{
    effdma_ctx->domain = domain;
    effdma_ctx->enabled = true;
}

void spdk_ext_io_effdma_clear(spdk_ext_io_effdma_ctx_t *effdma_ctx)
{
}
