#include <linux/virtio_config.h>
#include <linux/virtio_blk.h>

#include "config.h"
#include "virtio_blk_regs.h"
#include "snap_virtio_adm_spec.h"

void virtio_blk_regs_set(const struct virtio_blk_regs_set_attr *attr,
                          struct snap_virtio_blk_registers *regs)
{
    uint64_t device_features = 0;

    if (!attr->legacy_mode)
        device_features |= 1ULL << VIRTIO_F_VERSION_1;

    regs->queue_size = attr->queue_depth;
    regs->max_queues = attr->max_queues + (attr->max_queues > 0 ? attr->admin_queue : 0);
    device_features |= 1ULL << VIRTIO_BLK_F_MQ;

    regs->capacity = (attr->num_blocks * attr->blk_size) / 512;
    regs->size_max = attr->size_max ? : 0;
    device_features |= 1ULL << VIRTIO_BLK_F_SIZE_MAX;
    regs->seg_max = attr->seg_max ? : 0;
    device_features |= 1ULL << VIRTIO_BLK_F_SEG_MAX;
    regs->blk_size = attr->blk_size;
    device_features |= 1ULL << VIRTIO_BLK_F_BLK_SIZE;

    if (attr->admin_queue) {
      device_features |= 1ULL << VIRTIO_F_ADMIN_VQ;
      device_features |= 1ULL << VIRTIO_F_ADMIN_MIGRATION;
    }
    regs->device_features = device_features;
}
